-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: HPS")

L:RegisterTranslations("enUS", function() return {
	["Healing per Second"] = true,
	["HPS"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing per Second"] = "초당 치유량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil

local data = nil
local dirty = nil
local broadcastdirty = nil
local seconds = nil
local healingDone = nil
local moduleName = "HPS"
local isInCombat = {}
local combatStartTime = {}
local lastHealTime = {}

local COMBAT_TIMEOUT = 4

local roster = AceLibrary("Roster-2.1")
local parser = ParserLib:GetInstance("1.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 38177 $"):sub(12, -3)) or 1
mod.version = 4
mod.hidePercentage = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = mod:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType(moduleName, "heal", self.OnParserEvent)
	
	mod:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	mod:ScheduleRepeatingEvent( self.CheckPartyCombat, 1, self )
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes(moduleName )
end

function mod:OnReset()
	if type(data) == "table" then
		data = mod:del(data)
	end

	data = mod:new()
	data[playerName] = 0
	
	if type(seconds) == "table" then
		seconds = mod:del(seconds)
	end
	seconds = mod:new()
	
	if type(healingDone) == "table" then
		healingDone = mod:del(healingDone)
	end
	healingDone = mod:new()

	dirty = nil
	broadcastdirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Healing per Second"] end
function mod:GetShortDisplayName() return L["HPS"] end
function mod:GetBroadcastData()
	broadcastdirty = nil
	return data[playerName]
end

function mod:IsBroadcastDirty()
	return broadcastdirty
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

local function getDuration(name)
	local duration = seconds[name] or 0
	if combatStartTime[name] then
		duration = duration + math.max((lastHealTime[name] or GetTime()) - combatStartTime[name], 1.5)
	end
	return math.max(duration, 1.5)
end

function mod:OnRemoteData(sender, input)
	if data[sender] ~= input then
		data[sender] = input
		dirty = true
	end
end

function mod:ResetHPSTimer(name)
	seconds[name] = getDuration(name)
	
	combatStartTime[name] = nil
	lastHealTime[name] = nil
	
	mod:UpdateHPS(name)
end

function mod:PLAYER_REGEN_ENABLED()
	mod:ResetHPSTimer( playerName )
end

function mod:UpdateHPS(name)
	local newHPS = math.floor((healingDone[name] or 0) / getDuration(name))
	if name == playerName then
		if newHPS ~= data[name] then
			data[name], dirty, broadcastdirty = newHPS, true, true
		end
	else 
		mod:OnRemoteData( name, newHPS )
	end
end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function mod:OnParserEvent( info )
	local name, amount, victim = info.source, info.amount, info.victim

	if not name or name == ParserLib_NONE or not victim or victim == ParserLib_NONE or not amount or amount == 0 then
		return
	end
	
	if not Violation:AcceptDataFromSource( name, moduleName ) then
		return
	end

	local victimid
	if victimid == ParserLib_SELF then
		victimid = "player"
	else
		victimid = roster:GetUnitIDFromName( victim ) 
	end
	
	if name == ParserLib_SELF then
		name = playerName
	end
 	local newTime = GetTime()
	if lastHealTime[name] and lastHealTime[name] + COMBAT_TIMEOUT < newTime then
		mod:ResetHPSTimer(name)
	end
	
	combatStartTime[name] = combatStartTime[name] or newTime
	lastHealTime[name] = newTime

	amount = amount - getOverheal( victimid, amount )
	healingDone[name] = ( healingDone[name] or 0 ) + amount	
	
	mod:UpdateHPS(name)
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] and not UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = nil
			mod:ResetHPSTimer( name )
		elseif isInCombat[name] and lastHealTime[name] and lastHealTime[name] + COMBAT_TIMEOUT < GetTime() then
			mod:ResetHPSTimer(name)			
		elseif not isInCombat[name] and UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
